--
-- AdvancedFillAutoAimTargetNode
--
--  Author:  mjk, Jakob Tischler
--  Datum: 26.04.2015
--  Version: 1.3
--
-- Not for commercial use  
-- 

-- Anleitung:
-- Das Script macht alles allein. 
-- Für ein optimales Erebnis kann man in der Fahrzeug Xml folgenden Eintrag hinzufügen.
-- <advancedFillAutoAimTargetNode DistanceFront="2.5" DistanceBack="2.4" DistanceSide="0.8" Heightfix="0.2"/>

--Erklaerung:
-- DistanceFront="1.5" 			--> Wie weit die TargetNode nach vorne geht.
-- DistanceBack="1.7"  			--> Wie weit die TargetNode nach hinten geht.
-- DistanceSide="0.6"  			--> Wie weit sich die TargetNode nach links und rechts verschieben kann.
-- Heightfix="-0.2"				--> Verschiebt die TargetNode nach oben oder unten, falls eine Bordwand im Weg ist.
-- ShowNode="true/false"		--> Zeigt im Spiel die FillAutoAimTargetNode an.

-- Alle Werte sind optional!
-- Der Ausgangspunkt ist die eigentliche FillAutoAimTargetNode aus der Xml.


advancedFillAutoAimTargetNode = {};
local IsDebug = false;

local fillableChaff = false;
local fillableWoodChips = false;
local oldLoadInPercent = 0;

function advancedFillAutoAimTargetNode:prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations)and SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function advancedFillAutoAimTargetNode:load(xmlFile)
	
	if not hasXMLProperty(xmlFile, 'vehicle.fillAutoAimTargetNode') or self.capacity <= 0 or not self.fillAutoAimTargetNode then
		return;
	end;	

	self.advancedFillAutoAimTargetNode = {};	
	self.ShowNode = false;
	self.advancedFillAutoAimTargetNode.DefaultTrailers = {
		['data/vehicles/trailers/bergmann/bergmannCarex38S.i3d'] 				= {2.55, 2.20, 0.75, 0.1},
		['data/vehicles/trailers/bergmann/bergmannHTW65.i3d'] 					= {3.75, 3.25, 0.75, 0.3},
		['data/vehicles/trailers/brantner/brantnerE8041.i3d'] 					= {1.45, 1.45, 0.70, 0	},
		['data/vehicles/trailers/fliegl/flieglASS2101.i3d'] 					= {3.60, 3.80, 0.80, -0.3}, 
		['data/vehicles/trailers/fliegl/flieglTMK266Bull.i3d'] 					= {2.00, 1.90, 0.80, 0	}, 
		['data/vehicles/trailers/krampe/krampeBandit750.i3d'] 					= {2.65, 3.15, 0.80, 0.05}, 
		['data/vehicles/trailers/krampe/krampeBBS900.i3d'] 						= {3.50, 3.50, 0.80, 0.2},
		['data/vehicles/trailers/krampe/krampeSB3060.i3d'] 						= {3.80, 3.80, 0.80, 0.3}, 
		['data/vehicles/trailers/kroeger/kroegerHKD302.i3d'] 					= {2.05, 2.50, 0.80, 0.2}, 
		['data/vehicles/trailers/kroeger/kroegerTAW30.i3d'] 					= {3.40, 3.40, 0.80, 0.1}, 
		['data/vehicles/trailers/poettinger/poettingerEuroboss330T.i3d'] 		= {1.40, 1.40, 0.50, 0	},
		['$data/vehicles/trailers/strautmann/strautmannTeraVitesse5201.i3d'] 	= {3.20, 2.65, 0.80, 0.2},
		['KroneBigX1100BeastPack_BigXCargo.i3d'] 								= {0.0, 0.0, 0.0, 0.0},
		['MengeleGarant5402.i3d']												= {2.35, 2.15, 0.5, 0.0},		
	};

	local front, back, side, heightfix;	--default Values
	local x, y, z = getTranslation(self.fillAutoAimTargetNode);	
	local filename = getXMLString(xmlFile, "vehicle.filename");	
	
	if self.fillTypes[Fillable.FILLTYPE_CHAFF] then 		--Chaff
		fillableChaff = true;
	end;
	if self.fillTypes[Fillable.FILLTYPE_WOODCHIPS] then		--WoodChips
		fillableWoodChips = true;
	end;
	
	self.setFillLevel = Utils.appendedFunction(self.setFillLevel, advancedFillAutoAimTargetNode.SetAdvancedFillAutoAimTargetNode);

	if getXMLString(xmlFile, 'vehicle.advancedFillAutoAimTargetNode#DistanceFront') ~= nil then			--advancedFillAutoAimTargetNode was in Xmlfile found
		front = getXMLFloat(xmlFile, "vehicle.advancedFillAutoAimTargetNode#DistanceFront") or 1.5;
		back =  getXMLFloat(xmlFile, "vehicle.advancedFillAutoAimTargetNode#DistanceBack") or 1.5;
		side = getXMLFloat(xmlFile, "vehicle.advancedFillAutoAimTargetNode#DistanceSide") or 0.5;
		heightfix = getXMLFloat(xmlFile, "vehicle.advancedFillAutoAimTargetNode#Heightfix") or 0;
		self.ShowNode = getXMLBool(xmlFile, "vehicle.advancedFillAutoAimTargetNode#ShowNode") or false;
		
	else																								--search in Table for default Values
		for k,v in pairs(self.advancedFillAutoAimTargetNode.DefaultTrailers) do
			if k == filename then			
				front, back, side, heightfix = unpack(self.advancedFillAutoAimTargetNode.DefaultTrailers[filename]);	
			 	--if IsDebug then print(('advancedFillAutoAimTargetNode: Found defined Trailer (%q)	Values: Front:%.1f, Back=%.1f, Side=%.1f, Heightfix=%.1f'):format(filename or 'no name', front, back, side, heightfix)); end;
				break;
			end;
		end;
	end;
	
	if IsDebug then print(('advancedFillAutoAimTargetNode: Trailer %q was defined with: Front=%.1f, Back=%.1f, Side=%.1f, Heightfix=%.1f'):format(filename or 'no name', front, back, side, heightfix)); end;
	
	self.advancedFillAutoAimTargetNode.Positions = {
		[0] = {x 				,y 				,z 					},
		[1] = {x + side			,y + heightfix	,z + front			},
		[2] = {x + side			,y + heightfix	,z + front			},
		[3] = {x + side			,y + heightfix	,z + front			},
		[4] = {x + side			,y + heightfix	,z + front			},
		[5] = {x + side			,y + heightfix	,z + front			},
		[6] = {x 				,y + heightfix	,z + front			},
		[7] = {x 				,y + heightfix	,z + front			},
		[8] = {x 				,y + heightfix	,z + front			},
		[9] = {x 				,y + heightfix	,z + front			},
		[10] = {x 				,y + heightfix	,z + front			},
		[11] = {x - side		,y + heightfix	,z + front			},
		[12] = {x - side		,y + heightfix	,z + front			},
		[13] = {x - side		,y + heightfix	,z + front			},
		[14] = {x - side		,y + heightfix	,z + front			},
		[15] = {x - side		,y + heightfix	,z + front			},
		[16] = {x - side		,y + heightfix	,z + front * 0.5	},
		[17] = {x - side 		,y + heightfix	,z + front * 0.5	},
		[18] = {x - side * 0.5	,y + heightfix	,z + front * 0.5	},
		[19] = {x 				,y + heightfix	,z + front * 0.5	},
		[20] = {x 				,y + heightfix	,z + front * 0.5	},
		[21] = {x 				,y + heightfix	,z + front * 0.5	},
		[22] = {x 				,y + heightfix	,z + front * 0.5	},
		[23] = {x + side * 0.5	,y + heightfix	,z + front * 0.5	},
		[24] = {x + side		,y + heightfix	,z + front * 0.5	},
		[25] = {x + side		,y + heightfix	,z + front * 0.5	},
		[26] = {x + side		,y + heightfix	,z 					},
		[27] = {x + side 		,y + heightfix	,z 					},
		[28] = {x + side * 0.5	,y + heightfix	,z 					},
		[29] = {x 				,y + heightfix	,z 					},
		[30] = {x 				,y + heightfix	,z 					},
		[31] = {x 				,y + heightfix	,z 					},
		[32] = {x 				,y + heightfix	,z 					},
		[33] = {x - side * 0.5	,y + heightfix	,z 					},
		[34] = {x - side 		,y + heightfix	,z 					},
		[35] = {x - side 		,y + heightfix	,z 					},
		[36] = {x - side 		,y + heightfix	,z - back * 0.5		},
		[37] = {x - side 		,y + heightfix	,z - back * 0.5		},
		[38] = {x - side * 0.5	,y + heightfix	,z - back * 0.5		},
		[39] = {x  				,y + heightfix	,z - back * 0.5		},
		[40] = {x  				,y + heightfix	,z - back * 0.5		},
		[41] = {x  				,y + heightfix	,z - back * 0.5		},
		[42] = {x  				,y + heightfix	,z - back * 0.5		},
		[43] = {x + side * 0.5	,y + heightfix	,z - back * 0.5		},
		[44] = {x + side 		,y + heightfix	,z - back * 0.5		},
		[45] = {x + side 		,y + heightfix	,z - back * 0.5		},
		[46] = {x + side 		,y + heightfix	,z - back 			},
		[47] = {x + side 		,y + heightfix	,z - back 			},
		[48] = {x + side * 0.5	,y + heightfix	,z - back 			},
		[49] = {x 		 		,y + heightfix	,z - back 			},
		[50] = {x 				,y + heightfix	,z - back 			},
		[51] = {x  				,y + heightfix	,z - back 			},
		[52] = {x 				,y + heightfix	,z - back 			},
		[53] = {x - side * 0.5	,y + heightfix	,z - back 			},
		[54] = {x - side 		,y + heightfix	,z - back 			},
		[55] = {x - side 		,y + heightfix	,z - back 			},
		[56] = {x - side 		,y + heightfix	,z - back * 0.8		},
		[57] = {x - side 		,y + heightfix	,z - back * 0.6		},
		[58] = {x - side 		,y + heightfix	,z - back * 0.4		},
		[59] = {x - side 		,y + heightfix	,z - back * 0.2		},
		[60] = {x - side * 0.5	,y + heightfix	,z - back * 0.2		},
		[61] = {x - side * 0.5	,y + heightfix	,z - back * 0.4		},
		[62] = {x - side * 0.5	,y + heightfix	,z - back * 0.6		},
		[63] = {x - side * 0.5	,y + heightfix	,z - back * 0.8		},
		[64] = {x 				,y + heightfix	,z - back * 0.8		},
		[65] = {x 				,y + heightfix	,z - back * 0.6		},
		[66] = {x 				,y + heightfix	,z - back * 0.4		},
		[67] = {x  				,y + heightfix	,z - back * 0.2		},
		[68] = {x + side * 0.5 	,y + heightfix	,z - back * 0.2		},
		[69] = {x + side * 0.5	,y + heightfix	,z - back * 0.4		},
		[70] = {x + side * 0.5	,y + heightfix	,z - back * 0.6		},
		[71] = {x + side * 0.5 	,y + heightfix	,z - back * 0.8		},
		[72] = {x + side 	 	,y + heightfix	,z - back * 0.8		},
		[73] = {x + side		,y + heightfix	,z - back * 0.6		},
		[74] = {x + side 		,y + heightfix	,z - back * 0.4		},
		[75] = {x + side 		,y + heightfix	,z - back * 0.2		},
		[76] = {x + side 		,y + heightfix	,z + front * 0.8	},
		[77] = {x + side 		,y + heightfix	,z + front * 0.6	},
		[78] = {x + side 		,y + heightfix	,z + front * 0.4	},
		[79] = {x + side 		,y + heightfix	,z + front * 0.2	},
		[80] = {x + side * 0.5	,y + heightfix	,z + front * 0.2	},
		[81] = {x + side * 0.5	,y + heightfix	,z + front * 0.4	},
		[82] = {x + side * 0.5	,y + heightfix	,z + front * 0.6	},
		[83] = {x + side * 0.5	,y + heightfix	,z + front * 0.8	},
		[84] = {x 				,y + heightfix	,z + front * 0.8	},
		[85] = {x 				,y + heightfix	,z + front * 0.6	},
		[86] = {x 				,y + heightfix	,z + front * 0.4	},
		[87] = {x  				,y + heightfix	,z + front * 0.2	},
		[88] = {x - side * 0.5 	,y + heightfix	,z + front * 0.2	},
		[89] = {x - side * 0.5	,y + heightfix	,z + front * 0.4	},
		[90] = {x - side * 0.5	,y + heightfix	,z + front * 0.6	},
		[91] = {x - side * 0.5 	,y + heightfix	,z + front * 0.8	},
		[92] = {x - side 	 	,y + heightfix	,z + front * 0.8	},
		[93] = {x - side		,y + heightfix	,z + front * 0.6	},
		[94] = {x - side 		,y + heightfix	,z + front * 0.4	},
		[95] = {x - side 		,y + heightfix	,z + front * 0.2	},
		[96] = {x  				,y + heightfix	,z + front * 0.6	},
		[97] = {x  				,y + heightfix	,z + front * 0.3	},
		[98] = {x  				,y + heightfix	,z - back * 0.2		},
		[99] = {x  				,y + heightfix	,z - back * 0.6		},
		[100] = {x 				,y 				,z 					},	
	};	
	
end;

function advancedFillAutoAimTargetNode:SetAdvancedFillAutoAimTargetNode(fillLevel, fillType, force)		
	if fillType == Fillable.FILLTYPE_CHAFF and fillableChaff == true then 				--Chaff
	elseif fillType == Fillable.FILLTYPE_WOODCHIPS and fillableWoodChips == true then	--WoodChips 
	else return;																		-- everything else....return
	end;
	
	-- Load in %
	local loadInPercent =  math.floor(((self.fillLevel / self.capacity) * 100) + .1);
	
	if loadInPercent ~= oldLoadInPercent then
		local x, y, z = unpack(self.advancedFillAutoAimTargetNode.Positions[loadInPercent]);
		setTranslation(self.fillAutoAimTargetNode, x, y, z );
		oldLoadInPercent = loadInPercent;
	end;
end;

function advancedFillAutoAimTargetNode:update(dt)
	if (self.ShowNode and self.fillAutoAimTargetNode and self:getIsActive()) or (IsDebug and self.fillAutoAimTargetNode and self:getIsActive()) then
		local x,y,z = getWorldTranslation(self.fillAutoAimTargetNode);
		drawDebugPoint(x,y,z, 1, 0.08, 0.57, 1);
	end;
end;

function advancedFillAutoAimTargetNode:delete() end;
function advancedFillAutoAimTargetNode:mouseEvent(posX, posY, isDown, isUp, button) end;
function advancedFillAutoAimTargetNode:keyEvent(unicode, sym, modifier, isDown) end;
--function advancedFillAutoAimTargetNode:readStream(streamId, connection) end;
--function advancedFillAutoAimTargetNode:writeStream(streamId, connection)end;
function advancedFillAutoAimTargetNode:updateTick(dt) end;
function advancedFillAutoAimTargetNode:draw() end;